//
//  TWSOpenOptions.h
//  Twilio Sync Client
//
//  Copyright (c) 2017 Twilio, Inc. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "TWSConstants.h"

/** Options object specifying the open mode and parameters for a Sync entity. */
@interface TWSOpenOptions : NSObject

/** Create a new entity with the uniqueName if provided, return an error if it exists already.
 If no uniqueName is specified, this operation will always succeed provided sufficient permissions.
 
 @param uniqueName The optional unique name for the new entity or nil if no uniqueName is desired.
 @return The new TWSOpenOptions object.
 */
+ (nonnull TWSOpenOptions *)createWithUniqueName:(nullable NSString *)uniqueName;

/** Create a new or open an existing new entity with the uniqueName provided.
 This operation will always succeed provided sufficient permissions.
 
 @param uniqueName The non-optional unique name to either retrieve or create if needed.
 @return The new TWSOpenOptions object.
 */
+ (nullable TWSOpenOptions *)withUniqueName:(nonnull NSString *)uniqueName;

/** Open an existing new entity with the SID or uniqueName provided.
 
 @param sidOrUniqueName The non-optional SID or unique name to retrieve.
 @return The new TWSOpenOptions object.
 */
+ (nullable TWSOpenOptions *)openWithSidOrUniqueName:(nonnull NSString *)sidOrUniqueName;

/** Optionally specifies the synchronization strategy in use for pre-fetching objects, the default is
 TWSSynchronizationStrategyDefault.
 
 @param strategy The synchronization strategy to use, see TWSSynchronizationStrategy.
 @return The updated TWSOpenOptions object.
 */
- (nonnull TWSOpenOptions *)synchronizationStrategy:(TWSSynchronizationStrategy)strategy;

/** Optionally specifies the time to live for this object.
 
 @param The optional time to live for this item.
 @return The updated TWSOpenOptions object;
 */
- (nonnull TWSOpenOptions *)ttl:(TWSDuration)ttl;

/** The SID or uniqueName of the entity for the operation. */
- (nullable NSString *)sidOrUniqueName;

/** The synchronization strategy for the resultant entity. */
- (TWSSynchronizationStrategy)strategy;

/** The specified time to live, default is TWSDurationInfinity. */
- (TWSDuration)ttl;

@end
